@@ -48,7 +48,7 @@ using Banshee.PlaybackController;
 namespace Banshee.Collection.Database
 {
     public class DatabaseTrackListModel : TrackListModel, IExportableModel,
-        ICacheableDatabaseModel, IFilterable, ISortable, ICareAboutView
+        ICacheableDatabaseModel, IFilterable, ISortable, ICareAboutView, ISearchable
     {
         private readonly BansheeDbConnection connection;
         private IDatabaseTrackModelProvider provider;
@@ -58,7 +58,7 @@ namespace Banshee.Collection.Database
         private long count;
 
         private long filtered_count;
-        private TimeSpan filtered_duration;
+        private TimeSpan filtered_duration, duration;
         private long filtered_filesize, filesize;
 
         private ISortableColumn sort_column;
@@ -159,6 +159,13 @@ namespace Banshee.Collection.Database
             return true;
         }
 
+        public void Resort ()
+        {
+            var column = sort_column;
+            sort_column = null;
+            Sort (column);
+        }
+
         private void HandleCacheAggregatesUpdated (IDataReader reader)
         {
             filtered_duration = TimeSpan.FromMilliseconds (reader.IsDBNull (1) ? 0 : Convert.ToInt64 (reader[1]));
@@ -170,6 +177,7 @@ namespace Banshee.Collection.Database
             cache.Clear ();
             count = 0;
             filesize = 0;
+            duration = TimeSpan.Zero;
             filtered_count = 0;
             OnCleared ();
         }
@@ -200,7 +208,18 @@ namespace Banshee.Collection.Database
 
         private string from;
         protected string From {
-            get { return from ?? provider.From; }
+            get {
+                if (from == null) {
+                    from = provider.From;
+                    int i = from.IndexOf (',');
+                    if (i > 0) {
+                        // Force the join order to fix bgo#581103 and bgo#603661
+                        // See section 5.2 in http://www.sqlite.org/optoverview.html
+                        from = from.Substring (0, i) + " CROSS JOIN " + from.Substring (i + 1);
+                    }
+                }
+                return from;
+            }
             set { from = value; }
         }
 
@@ -212,12 +231,13 @@ namespace Banshee.Collection.Database
         public virtual void UpdateUnfilteredAggregates ()
         {
             HyenaSqliteCommand count_command = new HyenaSqliteCommand (String.Format (
-                "SELECT COUNT(*), SUM(CoreTracks.FileSize) {0}", UnfilteredQuery
+                "SELECT COUNT(*), SUM(CoreTracks.FileSize), SUM(CoreTracks.Duration) {0}", UnfilteredQuery
             ));
 
             using (HyenaDataReader reader = new HyenaDataReader (connection.Query (count_command))) {
                 count = reader.Get<long> (0);
                 filesize = reader.Get<long> (1);
+                duration = reader.Get<TimeSpan> (2);
             }
         }
 
@@ -319,6 +339,23 @@ namespace Banshee.Collection.Database
             cache.Reload ();
         }
 
+        private QueryFieldSet query_fields = BansheeQuery.FieldSet;
+        public QueryFieldSet QueryFields {
+            get { return query_fields; }
+            protected set { query_fields = value; }
+        }
+
+        public int IndexOf (QueryNode query, long offset)
+        {
+            lock (this) {
+                if (query == null) {
+                    return -1;
+                }
+
+                return (int) cache.IndexOf (query.ToSql (QueryFields), offset);
+            }
+        }
+
         public override int IndexOf (TrackInfo track)
         {
             lock (this) {
@@ -334,20 +371,20 @@ namespace Banshee.Collection.Database
         public int IndexOfFirst (TrackInfo track)
         {
             lock (this) {
-                return IndexOf (cache.GetSingle ("AND MetadataHash = ? ORDER BY OrderID", track.MetadataHash));
+                return IndexOf (cache.GetSingleWhere ("AND MetadataHash = ? ORDER BY OrderID", track.MetadataHash));
             }
         }
 
         public override TrackInfo GetRandom (DateTime notPlayedSince)
         {
-            return GetRandom (notPlayedSince, PlaybackShuffleMode.Song, true, false, Shuffler.Playback);
+            return GetRandom (notPlayedSince, "song", true, false, Shuffler.Playback);
         }
 
-        public TrackInfo GetRandom (DateTime notPlayedSince, PlaybackShuffleMode mode, bool repeat, bool resetSinceTime, Shuffler shuffler)
+        public TrackInfo GetRandom (DateTime notPlayedSince, string shuffle_mode, bool repeat, bool resetSinceTime, Shuffler shuffler)
         {
             lock (this) {
                 shuffler.SetModelAndCache (this, cache);
-                return shuffler.GetRandom (notPlayedSince, mode, repeat, resetSinceTime);
+                return shuffler.GetRandom (notPlayedSince, shuffle_mode, repeat, resetSinceTime);
             }
         }
 
@@ -363,11 +400,11 @@ namespace Banshee.Collection.Database
             get { return (int) filtered_count; }
         }
 
-        public TimeSpan Duration {
+        public virtual TimeSpan Duration {
             get { return filtered_duration; }
         }
 
-        public long FileSize {
+        public virtual long FileSize {
             get { return filtered_filesize; }
         }
 
@@ -375,6 +412,10 @@ namespace Banshee.Collection.Database
             get { return filesize; }
         }
 
+        public TimeSpan UnfilteredDuration {
+            get { return duration; }
+        }
+
         public int UnfilteredCount {
             get { return (int) count; }
             set { count = value; }
