@@ -37,7 +37,7 @@ using Mono.Unix;
 using Hyena;
 using Hyena.Query;
 using Hyena.Data.Sqlite;
- 
+
 using Banshee.Base;
 using Banshee.Query;
 using Banshee.Sources;
@@ -55,7 +55,7 @@ namespace Banshee.SmartPlaylist
     {
         private static List<SmartPlaylistSource> playlists = new List<SmartPlaylistSource> ();
         private static uint timeout_id = 0;
-        
+
         static SmartPlaylistSource () {
             Migrator.MigrateAll ();
 
@@ -65,13 +65,13 @@ namespace Banshee.SmartPlaylist
 
         private static string generic_name = Catalog.GetString ("Smart Playlist");
         private static string properties_label = Catalog.GetString ("Edit Smart Playlist");
-    
+
         private QueryOrder query_order;
         private QueryLimit limit;
         private IntegerQueryValue limit_value;
-        
+
         private List<SmartPlaylistSource> dependencies = new List<SmartPlaylistSource>();
-        
+
 
 #region Properties
 
@@ -116,15 +116,26 @@ namespace Banshee.SmartPlaylist
                 if (condition != null) {
                     condition_sql = condition.ToSql (BansheeQuery.FieldSet);
                     condition_xml = condition.ToXml (BansheeQuery.FieldSet);
-                    
+
                     foreach (QueryTermNode term in condition.GetTerms ()) {
                         if (!relevant_fields.Contains (term.Field))
                             relevant_fields.Add (term.Field);
                     }
+                } else {
+                    condition_sql = null;
+                    condition_xml = null;
                 }
             }
         }
 
+        private string additional_conditions;
+        public void AddCondition (string part)
+        {
+            if (!String.IsNullOrEmpty (part)) {
+                additional_conditions = additional_conditions == null ? part : String.Format ("{0} AND {1}", additional_conditions, part);
+            }
+        }
+
         private string condition_sql;
         public virtual string ConditionSql {
             get { return condition_sql; }
@@ -168,12 +179,18 @@ namespace Banshee.SmartPlaylist
                 return (Limit != null && LimitValue != null && !LimitValue.IsEmpty && QueryOrder != null);
             }
         }
-        
+
+        public bool IsHiddenWhenEmpty { get; private set; }
+
         public override bool HasDependencies {
             get { return dependencies.Count > 0; }
         }
 
         // FIXME scan ConditionTree for date fields
+        // FIXME even better, scan for date fields and see how fine-grained they are;
+        //       eg if set to Last Added < 2 weeks ago, don't need a timer going off
+        //       every 1 minute - every hour or two would suffice.  Just change this
+        //       property to a TimeSpan, rename it TimeDependentResolution or something
         public bool TimeDependent {
             get { return false; }
         }
@@ -182,43 +199,46 @@ namespace Banshee.SmartPlaylist
 
 #region Constructors
 
-        public SmartPlaylistSource (string name, int primarySourceId) : this (null, name, String.Empty, String.Empty, String.Empty, String.Empty, primarySourceId, 0)
+        public SmartPlaylistSource (string name, PrimarySource parent) : base (generic_name, name, parent)
+        {
+            SetProperties ();
+        }
+
+        public SmartPlaylistSource (string name, QueryNode condition, QueryOrder order, QueryLimit limit, IntegerQueryValue limit_value, PrimarySource parent)
+            : this (name, condition, order, limit, limit_value, false, parent)
         {
         }
 
-        public SmartPlaylistSource (string name, QueryNode condition, QueryOrder order, QueryLimit limit, IntegerQueryValue limit_value, int primarySourceId)
-            : base (generic_name, name, null, -1, 0, primarySourceId)
+        public SmartPlaylistSource (string name, QueryNode condition, QueryOrder order, QueryLimit limit, IntegerQueryValue limit_value, bool hiddenWhenEmpty, PrimarySource parent)
+            : this (name, parent)
         {
             ConditionTree = condition;
             QueryOrder = order;
             Limit = limit;
             LimitValue = limit_value;
+            IsHiddenWhenEmpty = hiddenWhenEmpty;
 
-            InstallProperties ();
+            SetProperties ();
             UpdateDependencies ();
         }
 
         // For existing smart playlists that we're loading from the database
-        protected SmartPlaylistSource (int? dbid, string name, string condition_xml, string order_by, string limit_number, string limit_criterion, int primarySourceId, int count) :
-            base (generic_name, name, dbid, -1, 0, primarySourceId)
+        protected SmartPlaylistSource (int dbid, string name, string condition_xml, string order_by, string limit_number, string limit_criterion, PrimarySource parent, int count, bool is_temp, bool hiddenWhenEmpty) :
+            base (generic_name, name, dbid, -1, 0, parent, is_temp)
         {
             ConditionXml = condition_xml;
             QueryOrder = BansheeQuery.FindOrder (order_by);
-            
             Limit = BansheeQuery.FindLimit (limit_criterion);
-
             LimitValue = new IntegerQueryValue ();
             LimitValue.ParseUserQuery (limit_number);
-
-            DbId = dbid;
             SavedCount = count;
+            IsHiddenWhenEmpty = hiddenWhenEmpty;
 
-            InstallProperties ();
-
+            SetProperties ();
             UpdateDependencies ();
         }
 
-        protected void InstallProperties ()
+        private void SetProperties ()
         {
             Properties.SetString ("Icon.Name", "source-smart-playlist");
             Properties.SetString ("SourcePropertiesActionLabel", properties_label);
@@ -240,17 +260,17 @@ namespace Banshee.SmartPlaylist
         }
 
 #endregion
-        
+
 #region Private Methods
-        
+
         private void UpdateDependencies ()
         {
             foreach (SmartPlaylistSource s in dependencies) {
                 s.Updated -= OnDependencyUpdated;
             }
-            
+
             dependencies.Clear ();
-            
+
             if (ConditionTree != null) {
                 foreach (SmartPlaylistQueryValue value in ConditionTree.SearchForValues<SmartPlaylistQueryValue> ()) {
                     SmartPlaylistSource playlist = value.ObjectValue;
@@ -261,27 +281,42 @@ namespace Banshee.SmartPlaylist
                 }
             }
         }
-        
+
         private void OnDependencyUpdated (object sender, EventArgs args)
         {
             Reload ();
         }
-        
+
 #endregion
 
 #region AbstractPlaylist overrides
 
+        protected override void AfterInitialized ()
+        {
+            base.AfterInitialized ();
+
+            if (PrimarySource != null) {
+                PrimarySource.TracksAdded += HandleTracksAdded;
+                PrimarySource.TracksChanged += HandleTracksChanged;
+                PrimarySource.TracksDeleted += HandleTracksDeleted;
+            }
+
+            if (IsHiddenWhenEmpty) {
+                RefreshAndReload ();
+            }
+        }
+
         protected override void Create ()
         {
             DbId = ServiceManager.DbConnection.Execute (new HyenaSqliteCommand (@"
                 INSERT INTO CoreSmartPlaylists
-                    (Name, Condition, OrderBy, LimitNumber, LimitCriterion, PrimarySourceID)
-                    VALUES (?, ?, ?, ?, ?, ?)",
+                    (Name, Condition, OrderBy, LimitNumber, LimitCriterion, PrimarySourceID, IsTemporary, IsHiddenWhenEmpty)
+                    VALUES (?, ?, ?, ?, ?, ?, ?, ?)",
                 Name, ConditionXml,
                 IsLimited ? QueryOrder.Name : null,
                 IsLimited ? LimitValue.ToSql () : null,
                 IsLimited ? Limit.Name : null,
-                PrimarySourceId
+                PrimarySourceId, IsTemporary, IsHiddenWhenEmpty
             ));
             UpdateDependencies ();
         }
@@ -295,13 +330,15 @@ namespace Banshee.SmartPlaylist
                         OrderBy = ?,
                         LimitNumber = ?,
                         LimitCriterion = ?,
-                        CachedCount = ?
+                        CachedCount = ?,
+                        IsTemporary = ?,
+                        IsHiddenWhenEmpty = ?
                     WHERE SmartPlaylistID = ?",
                 Name, ConditionXml,
                 IsLimited ? QueryOrder.Name : null,
                 IsLimited ? LimitValue.ToSql () : null,
                 IsLimited ? Limit.Name : null,
-                Count, DbId
+                Count, IsTemporary, IsHiddenWhenEmpty, DbId
             ));
             UpdateDependencies ();
         }
@@ -333,23 +370,39 @@ namespace Banshee.SmartPlaylist
         private bool refreshed = false;
         public override void Reload ()
         {
-            if (!refreshed)
+            if (!refreshed) {
                 Refresh ();
+            } else {
+                // Don't set this on the first refresh
+                Properties.Set<bool> ("NotifyWhenAdded", IsHiddenWhenEmpty);
+            }
 
             base.Reload ();
+
+            if (IsHiddenWhenEmpty && Parent != null) {
+                bool contains_me = Parent.ContainsChildSource (this);
+                int count = Count;
+
+                if (count == 0 && contains_me) {
+                    Parent.RemoveChildSource (this);
+                } else if (count > 0 && !contains_me) {
+                    Parent.AddChildSource (this);
+                }
+            }
         }
 
         public void Refresh ()
         {
-            // Wipe the member list clean and repopulate it 
+            // Wipe the member list clean and repopulate it
             string reload_str = String.Format (
                 @"DELETE FROM CoreSmartPlaylistEntries WHERE SmartPlaylistID = {0};
-                  INSERT INTO CoreSmartPlaylistEntries 
-                    SELECT NULL, {0} as SmartPlaylistID, TrackId
-                        FROM CoreTracks, CoreArtists, CoreAlbums
-                        WHERE CoreTracks.ArtistID = CoreArtists.ArtistID AND CoreTracks.AlbumID = CoreAlbums.AlbumID AND CoreTracks.PrimarySourceID = {1}
-                        {2} {3} {4}",
-                DbId, PrimarySourceId, PrependCondition("AND"), OrderSql, LimitSql
+                  INSERT INTO CoreSmartPlaylistEntries
+                    (EntryID, SmartPlaylistID, TrackID)
+                    SELECT NULL, {0} as SmartPlaylistID, TrackId FROM {1}
+                        WHERE {2} AND CoreTracks.PrimarySourceID = {3}
+                        {4} {5} {6}",
+                DbId, DatabaseTrackInfo.Provider.From, DatabaseTrackInfo.Provider.Where,
+                PrimarySourceId, PrependCondition ("AND"), OrderSql, LimitSql
             );
             ServiceManager.DbConnection.Execute (reload_str);
 
@@ -357,7 +410,7 @@ namespace Banshee.SmartPlaylist
             if (IsLimited && !Limit.RowBased) {
                 // Identify where the cut off mark is
                 HyenaSqliteCommand limit_command = new HyenaSqliteCommand (String.Format (
-                    @"SELECT EntryID, {0} 
+                    @"SELECT EntryID, {0}
                       FROM CoreTracks, CoreSmartPlaylistEntries
                       WHERE SmartPlaylistID = {1} AND CoreSmartPlaylistEntries.TrackID = CoreTracks.TrackID
                       ORDER BY EntryID",
@@ -405,18 +458,10 @@ namespace Banshee.SmartPlaylist
                 ));
             }
 
-            Remove ();
+            ThreadAssist.ProxyToMain (Remove);
             return true;
         }
 
-        public virtual bool CanUnmap {
-            get { return true; }
-        }
-
-        public bool ConfirmBeforeUnmap {
-            get { return true; }
-        }
-
         public bool CanRefresh {
             get { return QueryOrder == BansheeQuery.RandomOrder; }
         }
@@ -461,40 +506,39 @@ namespace Banshee.SmartPlaylist
         public override void SetParentSource (Source parent)
         {
             base.SetParentSource (parent);
-
-            PrimarySource primary = parent as PrimarySource;
-            if (primary != null) {
-                primary.TracksAdded += HandleTracksAdded;
-                primary.TracksChanged += HandleTracksChanged;
-                primary.TracksDeleted += HandleTracksDeleted;
-            }
         }
 
 #endregion
 
         private string PrependCondition (string with)
         {
-            return String.IsNullOrEmpty (ConditionSql) ? " " : String.Format ("{0} ({1})", with, ConditionSql);
+            string sql = String.IsNullOrEmpty (ConditionSql) ? " " : String.Format ("{0} ({1})", with, ConditionSql);
+            if (!String.IsNullOrEmpty (additional_conditions)) {
+                sql = String.IsNullOrEmpty (sql) ? additional_conditions : String.Format ("{0} AND ({1})", sql, additional_conditions);
+            }
+            return sql;
         }
 
-        public static IEnumerable<SmartPlaylistSource> LoadAll (int primary_id)
+        public static IEnumerable<SmartPlaylistSource> LoadAll (PrimarySource parent)
         {
-            using (IDataReader reader = ServiceManager.DbConnection.Query (
-                @"SELECT SmartPlaylistID, Name, Condition, OrderBy, LimitNumber, LimitCriterion, PrimarySourceID, CachedCount
-                    FROM CoreSmartPlaylists WHERE PrimarySourceID = ?", primary_id)) {
+            ClearTemporary ();
+            using (HyenaDataReader reader = new HyenaDataReader (ServiceManager.DbConnection.Query (
+                @"SELECT SmartPlaylistID, Name, Condition, OrderBy, LimitNumber, LimitCriterion, PrimarySourceID, CachedCount, IsTemporary, IsHiddenWhenEmpty
+                    FROM CoreSmartPlaylists WHERE PrimarySourceID = ?", parent.DbId))) {
                 while (reader.Read ()) {
                     SmartPlaylistSource playlist = null;
                     try {
                         playlist = new SmartPlaylistSource (
-                            Convert.ToInt32 (reader[0]), reader[1] as string,
-                            reader[2] as string, reader[3] as string,
-                            reader[4] as string, reader[5] as string,
-                            Convert.ToInt32 (reader[6]), Convert.ToInt32 (reader[7])
+                            reader.Get<int> (0), reader.Get<string> (1),
+                            reader.Get<string> (2), reader.Get<string> (3),
+                            reader.Get<string> (4), reader.Get<string> (5),
+                            parent, reader.Get<int> (7), reader.Get<bool> (8),
+                            reader.Get<bool> (9)
                         );
                     } catch (Exception e) {
                         Log.Warning ("Ignoring Smart Playlist", String.Format ("Caught error: {0}", e), false);
                     }
-                    
+
                     if (playlist != null) {
                         yield return playlist;
                     }
@@ -502,6 +546,16 @@ namespace Banshee.SmartPlaylist
             }
         }
 
+        private static void ClearTemporary ()
+        {
+            ServiceManager.DbConnection.Execute (@"
+                BEGIN TRANSACTION;
+                    DELETE FROM CoreSmartPlaylistEntries WHERE SmartPlaylistID IN (SELECT SmartPlaylistID FROM CoreSmartPlaylists WHERE IsTemporary = 1);
+                    DELETE FROM CoreSmartPlaylists WHERE IsTemporary = 1;
+                COMMIT TRANSACTION"
+            );
+        }
+
         private static void HandleSourceAdded (SourceEventArgs args)
         {
             SmartPlaylistSource playlist = args.Source as SmartPlaylistSource;
