@@ -30,6 +30,8 @@ using System;
 using Mono.Unix;
 using Gtk;
 
+using Hyena;
+
 using Banshee.Dap;
 using Banshee.Gui;
 using Banshee.ServiceStack;
@@ -47,12 +49,12 @@ namespace Banshee.Dap.Gui
         {
             AddImportant (
                 new ActionEntry ("SyncDapAction", null,
-                    Catalog.GetString ("Synchronize"), null,
+                    Catalog.GetString ("Sync"), null,
                     String.Empty, OnSyncDap)
             );
 
             AddUiFromFile ("GlobalUI.xml");
-            
+
             this["SyncDapAction"].IconName = Stock.Refresh;
             ServiceManager.SourceManager.ActiveSourceChanged += OnActiveSourceChanged;
             Actions.SourceActions.Updated += delegate { UpdateActions (); };
@@ -76,16 +78,14 @@ namespace Banshee.Dap.Gui
 
         private void OnSyncUpdated (DapSync sync)
         {
-            Banshee.Base.ThreadAssist.ProxyToMain (UpdateActions);
+            ThreadAssist.ProxyToMain (UpdateActions);
         }
 
         private void UpdateActions ()
         {
             DapSource dap = Dap;
             if (dap != null) {
-                UpdateAction ("SyncDapAction", dap.Sync.Enabled && !dap.Sync.AutoSync);
-                /* Translators: {0} is the name of the digital audio player */
-                this["SyncDapAction"].Label = String.Format (Catalog.GetString ("Synchronize {0}"), dap.Name);
+                UpdateAction ("SyncDapAction", dap.Sync.Enabled);
             }
         }
 
